bombgun = {}

function bombgun:new(parent, delay, speed)
	local table = {}
	table.__index = bombgun
	table.delay = delay
	table.bullets = {}
	table.timer = timer:new(table.delay, false)
	table.speed = speed
	table.damage = 3
	table.hitradius = 20
	table.parent = parent
	table.canshoot = false	
	table.keyfix = true
	table.img = love.graphics.newImage("res/bombbullet.png")
	table.flash = {}
	table.flash.img = love.graphics.newImage("res/gunflash.png")
	table.flash.width = table.flash.img:getWidth()
	table.flash.height = table.flash.img:getHeight()
	table.width = table.img:getWidth()
	table.height = table.img:getHeight()
	table.view = love.graphics.newImage("res/bombview.png")
	table.has = false
	setmetatable(table, table)
	return table
end

function bombgun:update(dt)
	for i = 1, #self.bullets do
		if self.bullets[i].active then
			if self.bullets[i].dir == "left" then
				self.bullets[i].x = self.bullets[i].x - self.speed * dt
			elseif self.bullets[i].dir == "right" then
				self.bullets[i].x = self.bullets[i].x + self.speed * dt
			elseif self.bullets[i].dir == "up" then
				self.bullets[i].y = self.bullets[i].y - self.speed * dt
			else
				self.bullets[i].y = self.bullets[i].y + self.speed * dt
			end
			if self.bullets[i].x > wx - camera.x or self.bullets[i].x < 0 - camera.x then
				self.bullets[i].active = false
			end
			if self.bullets[i].y > wy - camera.y or self.bullets[i].y < 0 - camera.y then
				self.bullets[i].active = false
			end

			if self.bullets[i].dir == "left" then
				self.bullets[i].hitbox = {x = self.bullets[i].x - self.bullets[i].width + self.bullets[i].width/2, y = self.bullets[i].y - self.bullets[i].height, width = self.bullets[i].width/2, height = self.bullets[i].height}
			elseif self.bullets[i].dir == "right" then
				self.bullets[i].hitbox = {x = self.bullets[i].x, y = self.bullets[i].y, width = self.bullets[i].width/2, height = self.bullets[i].height}
			elseif self.bullets[i].dir == "up" then
				self.bullets[i].hitbox = {x = self.bullets[i].x, y = self.bullets[i].y - self.bullets[i].height + self.bullets[i].height/2, width = self.bullets[i].width, height = self.bullets[i].height/2}
			else
				self.bullets[i].hitbox = {x = self.bullets[i].x, y = self.bullets[i].y, width = self.bullets[i].width, height = self.bullets[i].height/2}
			end
			
			self:collision(i)
		
			if self.bullets[i].active == false then
				self.bullets[i] = nil
			end
		
		end
	end

	ordertable(self.bullets)

	if self.canshoot == false then
		self.timer.active = true
	end
	if self.timer.tick then
		self.timer.active = false
		self.timer.tick = false
		self.canshoot = true
	end

end

function bombgun:draw()
	for i = 1, #self.bullets do
		if self.bullets[i].active then
			local scax = 1
			local scay = 1
			local rad = 0
			if self.bullets[i].dir == "left" then
				scax = -1
				scay = -1
			elseif self.bullets[i].dir == "right" then
				scax = 1
			elseif self.bullets[i].dir == "down" then
				rad = -math.pi/2
				scax = -1
				scay = 1
			else 
				rad = math.pi/2
				scax = -1
				scay = -1
			end
			love.graphics.draw(self.img, self.bullets[i].x, self.bullets[i].y,rad,scax,scay)
		
			if self.bullets[i].flash then
				local x,y
				if self.bullets[i].dir == "left" then
					scax = -1
					scay = -1	
					x = self.bullets[i].x - self.width
					y = self.bullets[i].y + self.flash.height/2 - self.height/2
				elseif self.bullets[i].dir == "right" then
					scax = 1
					x = self.bullets[i].x + self.flash.width/2 - self.width/2
					y = self.bullets[i].y - self.flash.height/2 + self.height/2
				elseif self.bullets[i].dir == "down" then
					rad = -math.pi/2
					scax = -1
					scay = 1
					x = self.bullets[i].x - self.flash.width/2 + self.width/2
					y = self.bullets[i].y + self.height
				else 
					rad = math.pi/2
					scax = -1
					scay = -1
					x = self.bullets[i].x - self.flash.width/2 + self.width/2
					y = self.bullets[i].y - self.flash.height/2 - self.height/2
				end
				love.graphics.draw(self.flash.img, x, y,rad,scax,scay)
				self.bullets[i].flash = false
			end

			if gamedebug then
				love.graphics.rectangle("line", self.bullets[i].hitbox.x, self.bullets[i].hitbox.y, self.bullets[i].hitbox.width, self.bullets[i].hitbox.height)
				if self.bullets[i].dir == "left" then
					love.graphics.rectangle("line",self.bullets[i].x - self.hitradius - self.width/2,self.bullets[i].y - self.hitradius - self.height,2*self.hitradius + self.width,2*self.hitradius + self.height)
				elseif self.bullets[i].dir == "right" then
					love.graphics.rectangle("line",self.bullets[i].x - self.hitradius ,self.bullets[i].y - self.hitradius,2*self.hitradius + self.width,2*self.hitradius + self.height)
				elseif self.bullets[i].dir == "up" then
					love.graphics.rectangle("line",self.bullets[i].x - self.hitradius ,self.bullets[i].y - self.hitradius - self.bullets[i].height/2,2*self.hitradius + self.width,2*self.hitradius + self.height)
				else
					love.graphics.rectangle("line",self.bullets[i].x - self.hitradius ,self.bullets[i].y - self.hitradius,2*self.hitradius + self.width,2*self.hitradius + self.height)
				end
			end
		end
	end
end

function bombgun:shoot()
	if self.canshoot then
		local tab = {}
		tab.active = true
		tab.dir = self.parent.dir
		tab.flash = true
		if self.parent.look == "rect" then
			tab.width = self.width
			tab.height = self.height
			tab.x = self.parent.x + self.parent.width/2 + tab.width/2
			tab.y = self.parent.y + self.parent.height/2 - tab.height/2
			if tab.dir == "left" then
				tab.x = self.parent.x + self.parent.width/2 - tab.width/2
				tab.y = tab.y + tab.height
				tab.x = tab.x + tab.width
			end
		else
			tab.width = self.height
			tab.height = self.width
			tab.x = self.parent.x + self.parent.width/2
			tab.y = self.parent.y + self.parent.height/2 - tab.height/2
			if self.parent.look == "up" then
				tab.dir = "up"	
				tab.y = tab.y + tab.height
			else
				tab.dir = "down"
			end
		end
		self.bullets[#self.bullets + 1] = tab
		self.canshoot = false
		self.keyfix = false
	end
end

function bombgun:collision(i)
	local hit = false
	local tile = map:getneartilepos(self.bullets[i].hitbox.x, self.bullets[i].hitbox.y, self.bullets[i].hitbox.width, self.bullets[i].hitbox.height, currentmap)	
	for k = 1, #tile do
		for y = 1, #solidids do
			if tile[k].id == solidids[y] then
				if shortaabb(self.bullets[i].hitbox, tile[k]) then
					if self.bullets[i].dir == "left" then
						self.bullets[i].hitbox = {x = self.bullets[i].x - self.hitradius - self.width/2, y = self.bullets[i].y - self.hitradius - self.height, width = 2*self.hitradius + self.width, height = 2*self.hitradius + self.height}
					elseif self.bullets[i].dir == "right" then
						self.bullets[i].hitbox = {x = self.bullets[i].x - self.hitradius , y = self.bullets[i].y - self.hitradius, width = 2*self.hitradius + self.width, height = 2*self.hitradius + self.height}
					elseif self.bullets[i].dir == "up" then
						self.bullets[i].hitbox = {x = self.bullets[i].x - self.hitradius , y = self.bullets[i].y - self.hitradius - self.bullets[i].height/2, width = 2*self.hitradius + self.width, height = 2*self.hitradius + self.height}
					else
						self.bullets[i].hitbox = {x = self.bullets[i].x - self.hitradius , y = self.bullets[i].y - self.hitradius, width = 2*self.hitradius + self.width, height = 2*self.hitradius + self.height}
					end
					hit = true

					if tile[k].id == wallid then
						currentmap[tile[k].mapy][tile[k].mapx] = 0
						flashes[#flashes+1] = {x = tile[k].x + tilesize/2, y = tile[k].y + tilesize/2}
						particles[#particles +1] = partsys:new(smokeimg,tile[k].x + tilesize/2, tile[k].y + tilesize/2,{0.4,0.8},{tilesize,2 * tilesize}, math.random(3,7))
					end
				end
			end
		end
		
	end
	---- enemy collision
	for y = 1, #enemy do
		for k = 1, #enemy[y] do
			if enemy[y][k].active then
				if hit then
					if shortaabb(enemy[y][k].hitbox ,self.bullets[i].hitbox) then
						local dragonfire = false

						if enemy[y][k].firetimer ~= nil then -- dragon
							if enemy[y][k].firetimer.active then
								dragonfire = not enemy[y][k].hitboxchange	
							end
						end
			
						if dragonfire == false then
							enemy[y][k].health = enemy[y][k].health - self.damage
							enemy[y][k].hit = true
							if enemy[y][k].health <= 0 then
								enemy[y][k].dead = true
							end	
						end
					end
					
					for v = 1, #tile do
						if tile[v].id == wallid then
							currentmap[tile[v].mapy][tile[v].mapx] = 0
							flashes[#flashes+1] = {x = tile[v].x + tilesize/2, y = tile[v].y + tilesize/2}
							particles[#particles +1] = partsys:new(smokeimg,tile[v].x + tilesize/2, tile[v].y + tilesize/2,{0.4,0.8},{tilesize,2 * tilesize}, math.random(3,7))
						end
					end
				else
					if shortaabb(enemy[y][k].hitbox ,self.bullets[i].hitbox) then
						if self.bullets[i].dir == "left" then
							self.bullets[i].hitbox = {x = self.bullets[i].x - self.hitradius - self.width/2, y = self.bullets[i].y - self.hitradius - self.height, width = 2*self.hitradius + self.width, height = 2*self.hitradius + self.height}
						elseif self.bullets[i].dir == "right" then
							self.bullets[i].hitbox = {x = self.bullets[i].x - self.hitradius , y = self.bullets[i].y - self.hitradius, width = 2*self.hitradius + self.width, height = 2*self.hitradius + self.height}
						elseif self.bullets[i].dir == "up" then
							self.bullets[i].hitbox = {x = self.bullets[i].x - self.hitradius , y = self.bullets[i].y - self.hitradius - self.bullets[i].height/2, width = 2*self.hitradius + self.width, height = 2*self.hitradius + self.height}
						else
							self.bullets[i].hitbox = {x = self.bullets[i].x - self.hitradius , y = self.bullets[i].y - self.hitradius, width = 2*self.hitradius + self.width, height = 2*self.hitradius + self.height}
						end	
						if shortaabb(enemy[y][k].hitbox ,self.bullets[i].hitbox) then
							local dragonfire = false

							if enemy[y][k].firetimer ~= nil then -- dragon
								if enemy[y][k].firetimer.active then
									dragonfire = not enemy[y][k].hitboxchange	
								end
							end
		
							if dragonfire == false then
								enemy[y][k].health = enemy[y][k].health - self.damage
								enemy[y][k].hit = true
								if enemy[y][k].health <= 0 then
									enemy[y][k].dead = true
								end	
							end
						end

						for v = 1, #tile do
							if tile[v].id == wallid then
								currentmap[tile[v].mapy][tile[v].mapx] = 0
								flashes[#flashes+1] = {x = tile[v].x + tilesize/2, y = tile[v].y + tilesize/2}
								particles[#particles +1] = partsys:new(smokeimg,tile[v].x + tilesize/2, tile[v].y + tilesize/2,{0.4,0.8},{tilesize,2 * tilesize}, math.random(3,7))
							end
						end
						hit = true
						
					end	
				end
			end
		end
	end
	if hit then
		camera:shake(2,0.8)
		self.bullets[i].active = false
		local diff = 15
		for k = 1,4 do
			particles[#particles +1] = partsys:new(explosionimg,math.random(self.bullets[i].x - diff,self.bullets[i].x + diff ),math.random(self.bullets[i].y - diff,self.bullets[i].y + diff ),{0.4,0.8},{tilesize,1.5 * tilesize}, 3)
		end
		
		for k = 1,3 do
			flashes[#flashes+1] = {x = math.random(self.bullets[i].x - diff,self.bullets[i].x + diff ), y = math.random(self.bullets[i].y - diff,self.bullets[i].y + diff )}
		end
	end
end
