
blob = {}
blob.__index = blob

function blob:init()
	self.blobs = {}
	self.id = 7
	self.speed = 25
	self.jumpspeed = 120
	self.widthdif = 4
	self.heightdif = 4
	self.health = 3
	self.points = 100
	self.range = 3
	self.damage = 1
	self.type = "minion"
	self.img = love.graphics.newImage("res/blob.png")

	self.width = tilesize - self.widthdif
	self.height = tilesize - self.heightdif
	self.xoffset = self.widthdif/2
	self.yoffset = self.heightdif/2
	
	local pos = map:gettilepos(currentmap, {self.id})
	for i = 1, #pos do
		self.blobs[i] = {}
		self.blobs[i].x = pos[i].x
		self.blobs[i].y = pos[i].y	
		self.blobs[i].hitbox = {x = self.blobs[i].x + self.xoffset, y = self.blobs[i].y + self.yoffset, width = self.width, height = self.height}
		self.blobs[i].velx = 0	
		self.blobs[i].vely = 0
		self.blobs[i].state = "run"
		self.blobs[i].fall = true
		self.blobs[i].dir = "left"
		self.blobs[i].points = self.points
		self.blobs[i].health = self.health
		self.blobs[i].damage = self.damage
		self.blobs[i].onground = false
		self.blobs[i].hit = false
		self.blobs[i].anim = sprite:new(self.img, tilesize, tilesize, {{"stand",1}, {"run", 2},{"jump",1},{"hurt",1}},0,self.yoffset/2)
		self.blobs[i].active = false
		self.blobs[i].dead = false
		currentmap[pos[i].mapy][pos[i].mapx] = 0
	end
	setmetatable(blob, blob)
	enemy[#enemy + 1] = self.blobs --- remember this for future enemiessb
end

function blob:update(dt, i)

	self:move(dt, i)
	self:collision(i)

	if self.blobs[i].state == "hurt" then
		self.blobs[i].hit = false
	end

	if self.blobs[i].hit then
		self.blobs[i].state = "hurt"
		self.blobs[i].onground = false
		self.blobs[i].vely = -self.jumpspeed/2
		if self.blobs[i].x > player.x then
			self.blobs[i].velx = self.speed
			self.blobs[i].dir = "left"
		else
			self.blobs[i].velx = -self.speed
			self.blobs[i].dir = "right"
		end
	end

	if self.blobs[i].onground then
		if getdist(self.blobs[i].x - self.width/2, self.blobs[i].y - self.height/2, player.x - player.width/2, player.y - player.height/2) < self.range * tilesize then
			if player.x > self.blobs[i].x then -- player to the right
				self.blobs[i].onground = false
				self.blobs[i].velx = self.speed  * 2
				self.blobs[i].vely = -self.jumpspeed 
				self.blobs[i].dir = "right"
				self.blobs[i].state = "jump"
			else
				self.blobs[i].onground = false
				self.blobs[i].velx = -self.speed * 2
				self.blobs[i].vely = -self.jumpspeed
				self.blobs[i].dir = "left"
				self.blobs[i].state = "jump"
	
			end
		end
	end

end

function blob:loop()
	local dt = love.timer.getDelta()
	for i = 1, #self.blobs do
		if self.blobs[i].active then
			self:update(dt, i)
			if gamedebug == true then
				love.graphics.setColor(255, 0, 0)
				love.graphics.rectangle("line", self.blobs[i].hitbox.x, self.blobs[i].hitbox.y, self.blobs[i].hitbox.width, self.blobs[i].hitbox.height)
				love.graphics.setColor(255,255,255)
			end
			self.blobs[i].anim:draw(self.blobs[i], 0.2)
			self.blobs[i].hit = false
		end
	end
end

function blob:move(dt, i)
	self.blobs[i].y = (self.blobs[i].y + self.blobs[i].vely * dt)
	self.blobs[i].x = (self.blobs[i].x + self.blobs[i].velx * dt)

	self.blobs[i].vely = self.blobs[i].vely + gravity * dt

	if self.blobs[i].onground then
		if self.blobs[i].dir == "left" then
			self.blobs[i].velx = self.blobs[i].velx + self.speed * dt
			if self.blobs[i].velx > -5 then
				self.blobs[i].velx = -self.speed *1.5
			end
		else
			self.blobs[i].velx = self.blobs[i].velx - self.speed  * dt
			if self.blobs[i].velx < 5 then
				self.blobs[i].velx = self.speed *1.5
			end
		end
	end

	if self.blobs[i].vely ~= 0 then
		self.blobs[i].fall = true
		self.blobs[i].onground = false
	else
		self.blobs[i].fall = false
	end
	

end

function blob:collision(i)
	self.blobs[i].hitbox = {x = self.blobs[i].x + self.xoffset, y = self.blobs[i].y + self.yoffset, width = self.width, height = self.height}
	local tile = map:getneartilepos(self.blobs[i].x, self.blobs[i].y, self.width, self.height, currentmap)	
	for y = 1, #tile do
		for k = 1, #solidids do
			if tile[y].id == solidids[k] then
				local col = getcollision(self.blobs[i].hitbox, tile[y], 3)
				if col then
					if col.hor then
						self.blobs[i].x = self.blobs[i].x + col.x 
						self.blobs[i].velx = 0
					end
					if col.ver then
						self.blobs[i].y = self.blobs[i].y + col.y
						self.blobs[i].vely = 0
						self.blobs[i].fall = false
					end
					if col.side == "right" then
						self.blobs[i].dir = "left"
					end
					if col.side == "left" then
						self.blobs[i].dir = "right" 
					end
					if col.side == "down" then
						if self.blobs[i].onground == false then
							self.blobs[i].velx = 0
						end
						self.blobs[i].onground = true
						self.blobs[i].state = "run"
					end
				self.blobs[i].hitbox = {x = self.blobs[i].x + self.xoffset, y = self.blobs[i].y + self.yoffset, width = self.width, height = self.height}
				end
			end
		end
	end
end

