-- MAP DRAWER (1.1)
local map = {}

function map:init(tile)
	local scale = scale or 1
	map._wx = love.graphics.getWidth()/scale
	map._wy = love.graphics.getHeight()/scale
	self.tile = {}
	self.tile.img = love.graphics.newImage(tile)
	map._newquads(self.tile.img)

end

function map:draw(m, camerax, cameray) -- draws only tiles onscreen
	local camx = camerax or 0
	local camy = cameray or 0
	for y = math.floor(-camy/tilesize), map._wy/tilesize - camy/tilesize + 1 do
		for x = math.floor(-camx/tilesize) , map._wx/tilesize - camx/tilesize + 1 do
			if m[y] == nil then m[y] = {} end
			if m[y][x] == nil then m[y][x] = 0 end
			love.graphics.draw(self.tile.img, self.tile[m[y][x]], ((x - 1) * tilesize), ((y - 1) * tilesize))
		end
	end
end

function map._newquads(img) --guarda tiles de izquierda a derecha y luego hace lo mismo en la proxima fila. Todo esto depende del tamaño de la imagen
	local count = 0
	for m = 0, (img:getHeight() / tilesize) -1 do
		for n = 0, (img:getWidth() / tilesize) -1 do
			map.tile[count] = love.graphics.newQuad(n * tilesize, m * tilesize, tilesize, tilesize, img:getWidth(), img:getHeight())
			count = count + 1
		end	
	end
end

function map:gettilepos(m, idstable) -- m = textmap, idstable = a table with the numbers of the tile that want to we get.
	local n = 1
	local table = {}	
	for yi = 1, #m do
		for xi = 1, #m[yi] do
			for num = 1, #idstable do
				if m[yi][xi] == idstable[num] then
					table[n] = {}
					table[n].x = ((xi - 1) * tilesize)
					table[n].y = ((yi - 1) * tilesize)
					table[n].width = tilesize
					table[n].height = tilesize
					table[n].mapx = xi
					table[n].mapy = yi
					table[n].id = m[yi][xi]
					n = n + 1
				end
			end
		end
	end
	return table
end

function map:getneartilepos(x,y,width,height,map)
	local tab = {}
	local maxsize = math.ceil(math.max(width,height)/16)
	
	local posx = math.floor(x/tilesize)
	local posy = math.floor(y/tilesize)
	local sizewidth = 1+maxsize
	local sizeheight = 1+maxsize 
	local n = 1

	for my = 1, sizeheight do
		for mx = 1, sizewidth do
			tab[n] = {}
			tab[n].mapx = (posx + (mx-1)) + 1
			tab[n].mapy = (posy + (my-1)) + 1
			if map[tab[n].mapy] then
				tab[n].id = map[tab[n].mapy][tab[n].mapx]
			else
				tab[n].id = 0
			end
			tab[n].x = (tab[n].mapx-1) * tilesize
			tab[n].y = (tab[n].mapy-1) * tilesize
			tab[n].width = tilesize
			tab[n].height = tilesize
			n = n + 1
		end
	end
	return tab
end

function map:transtiled(tab, width) --Tiled maps to compatible maps
	local newtab = {{}}
	local y = 1
	local w = width
	local x = 1
	for i = 1, #tab do
		if tab[i] ~= 0 then
			tab[i] = tab[i] - 1
		end
		if i > w then
			w = w + width
			y = y + 1
			x = 1
			newtab[y] = {}
		end
		newtab[y][x] = tab[i]	
		x = x + 1	
	end
	return newtab
end

return map

