local system = {}
system.__index = system

--[[ 
img = sprite to us, each tile has to be a square, tiles right next to the other horizontally and in order
x = x position of the start
y = y position of the start
delay(speed) = time to reach the target range, it can be a table for random (ex: {1, 4} == math.random(1,4) ) 
range = the distance each particle has to travel, it can be a table for random (ex: {1, 4} == math.random(1,4) ) 
num = how many particles
angle = whickh dirrection the particle has to go, if nil, then it will be random, if table, then each element will be an angle and "num" will be the nambers of elements in the table 


]]

function system:new(img, x, y, delay, range,num, angle)
	local num = num or 1
	local tilesize = img:getHeight()
	local world = {}
	world.img = img
	world.imgwidth, world.imgheight = img:getWidth(),tilesize
	world.done = false
	world.obj = {}
	if angle then
		num = 1
		if type(angle) == "table" then
			num = #angle
		end
	end

	for k = 1, num do
		local obj = {}
		obj.x = x
		obj.y = y
		obj.velx = 0
		obj.vely = 0
		obj.active = true
		obj.timer = {}
		obj.timer.time = 0
		if type(range) == "table" then
			obj.range = math.random(range[1]*1000,range[2]*1000)/1000
		else
			obj.range = range
		end
		if angle then
			if type(angle) == "table" then
				obj.angle = -math.rad(angle[k])
			else
				obj.angle = -math.rad(angle)
			end
		else
			obj.angle = -math.rad(-math.random(0, 360))
		end
		obj.quads = {}
		for i = 1, (world.imgwidth/tilesize) do
			obj.quads[i] = love.graphics.newQuad((i-1)*tilesize, 0, tilesize, tilesize, world.imgwidth, tilesize)

		end

		if type(delay) == "table" then
			obj.timer.step = (math.random(delay[1]*1000,delay[2]*1000)/1000)/#obj.quads
		else
			obj.timer.step = delay/#obj.quads
		end

		obj.frame = 1
		world.obj[k] = obj
	end
	setmetatable(world, self)
	return world

end

function system:draw()
	local dt = love.timer.getDelta()
	
	local done = true

	for i = 1, #self.obj do
		local v = self.obj[i]	
		if v.active then
			done = false	

			v.timer.time = v.timer.time + dt

			if v.timer.time > v.timer.step then
				v.frame = v.frame + 1
				v.timer.time = 0
			end

			if v.frame > #v.quads then
				v.active = false
			end
			--movement
			local delay = v.timer.step * #v.quads
			local speed = v.range/delay

			v.x = v.x + math.cos(v.angle)*dt*speed
			v.y = v.y + math.sin(v.angle)*dt*speed
			---- DRAW	

			if v.active then
				love.graphics.draw(self.img, v.quads[v.frame],v.x, v.y)	
			end
		end
	end
	if done then
		self.done = true
	end
	
end

return system
