-- GENERAL USE PHYSICS ENGINE (1.1) FEAT. RAYCASTING AND BASE64 DECODER AND TABLE ORDERING	

--[[
	http://www.base64-image.de/
	image to base64 code
]]

function ordertable(t) -- removes nil/holes values
	local n = 0
	local lasti = 0
	for i,v in pairs(t) do
		if i > lasti then
			lasti = i
		end
	end
	for i = 1, lasti do
		if t[i] == nil then
			n = n + 1
		end
	end
	for y = 1, n do
		for i = 1, lasti do
			if t[i] == nil then
				t[i] = t[i + 1]
				t[i + 1] = nil
			end	
		end
	end
end

function base64decode(base64code)
	return love.graphics.newImage(love.image.newImageData(love.filesystem.newFileData(base64code, "", "base64")))
end


function aabb(ax1,ay1,aw,ah, bx1,by1,bw,bh)
	local ax2,ay2,bx2,by2 = ax1 + aw, ay1 + ah, bx1 + bw, by1 + bh
	return ax1 < bx2 and ax2 > bx1 and ay1 < by2 and ay2 > by1
end

function shortaabb(a,b)
	return aabb(a.x, a.y, a.width, a.height, b.x, b.y, b.width, b.height)
end

function getsides(ax1,ay1,aw,ah, bx1,by1,bw,bh)-- thus this work??
	if aabb(ax1,ay1,aw,ah, bx1,by1,bw,bh) then
		local onx = ax1 < bx1 and 'right' or (ax1 > bx1 and 'left' or 'superposed')
		local ony = ay1 < by1 and 'down' or (ay1 > by1 and 'up' or 'superposed')
		return onx,ony
	end
end

function pixelcollision(a, x, y) -- WITH OBJECTS(shortaabb style) -- needed for pix collision(keep scrolling)
	local ax2,ay2,bx2,by2 = a.x + a.width, a.y + a.height, x + 1, y + 1
	return a.x < bx2 and ax2 > x and a.y < by2 and ay2 > y
end

function getdist(x1, y1, x2, y2)
	dx = x2 - x1
	dy = y2 - y1
	return math.sqrt((dx^2 + dy^2))
end

-- a really good and simple collision detector (only for rectangles) 
-- pressision is to tweak how strict-less the detection has to be, cause a very strict detection can produce some problems

function getcollision(a, b, pressision) 
	if aabb(a.x, a.y, a.width, a.height, b.x, b.y, b.width, b.height) then
		local left = b.x - (a.x + a.width)
		local right = (b.x + b.width) - a.x
		local up = b.y - (a.y + a.height)
		local down = (b.y + b.height) - a.y

		local pressision = pressision or 2
		local table = {}

		if math.abs(left) < right then 
			table.x = left
			table.xside = "right"
		else
			table.x = right
			table.xside = "left"
		end

		if math.abs(up) < down then
			table.y = up
			table.yside = "down"
		else
			table.y = down
			table.yside = "up"
		end 
		if math.abs(table.x) < pressision and math.abs(table.y) < pressision and math.abs(table.x - table.y) < pressision then -- adds pressision
			return false
		end
		if math.abs(table.x) < math.abs(table.y) then
			table.y = 0
			table.side = table.xside
			table.hor = true
		else
			table.x = 0
			table.side = table.yside
			table.ver = true
		end
	
		return table
	else
		return false
	end

end

--[[function collide(a,b) --------------- EXAMPLE ON HOW TO USE IT
	local col = getcollision(a,b)
	if col then
		if col.hor then
			a.x = a.x + col.x 
			a.velx = 0
		end
		if col.ver then
			a.y = a.y + col.y
			a.vely = 0
		end
		if col.side = "down" then
			a.canjump = true
		end
	end
end]]

function setbounds(obj, x, tilesize, y) -- restrict the player movement
	local y = y or x
	local tilesize = tilesize or 16
	if obj.x < x * tilesize then
		obj.x = x * tilesize
	elseif obj.x + obj.width > wx - (x * tilesize) then
		obj.x = (wx - x * tilesize) - obj.width
	end

	if obj.y < y * tilesize then
		obj.y = y * tilesize
	elseif obj.y + obj.height > wy - y * tilesize   then
		obj.y = (wy - y * tilesize) - obj.height
	end
end

-- This is the second method to detect collision:
-- tweak is for strict-less
function pixdowncol(obj, target, tweak) -- if "down" collision
	local tweak = tweak or 2
	if pixelcollision(target, obj.x + tweak, obj.y + obj.height) or pixelcollision(target, obj.x + obj.width - tweak, obj.y + obj.height) then
		return true
	else
		return false
	end
end

function pixupcol(obj, target, tweak) -- if "up" collision
	local fix = 1
	local tweak = tweak or 2  
	if pixelcollision(target, obj.x + tweak, obj.y - fix) or pixelcollision(target, obj.x + obj.width - tweak, obj.y - fix) then
		return true
	else 
		return false
	end
end

function pixrightcol(obj, target, tweak) -- if "right" collision
	local tweak = tweak or 2
	if pixelcollision(target, obj.x + obj.width, obj.y + tweak) or pixelcollision(target, obj.x + obj.width, obj.y + obj.height - tweak) then
		return true
	else 
		return false
	end
end

function pixleftcol(obj, target, tweak) -- if "left" collision
	local fix = 1
	local tweak = tweak or 2 
	if pixelcollision(target, obj.x - fix, obj.y + tweak) or pixelcollision(target, obj.x - fix, obj.y + obj.height - tweak) then
		return true
	else 
		return false
	end
end

-- solidstable = table with solid tile num of the map
-- ww wh = window width and height
-- texturetable = a table with images in the order of the solids, it can be false
function drawraycast(x, y, dir, map, fov, solidstable, ww, wh, texturetable, tilesize, skip) 	
	local tilesize = tilesize or 1
	local wx = ww or love.graphics.getWidth()
	local wy = wh or love.graphics.getHeight()
	local fov = fov or 0.6
	local skip = skip or 1
	local texture = texturetable
	for i = 1, wx, skip do
		local dif = 2 * (i/wx) - 1 
		local raydir = dir + (fov * dif)
		local mapx = math.floor(x/tilesize)
		local mapy = math.floor(y/tilesize)
		local nextdistx = math.sqrt(1 + ((math.sin(raydir))^2 / (math.cos(raydir))^2))
		local nextdisty = math.sqrt(1 + ((math.cos(raydir))^2 / (math.sin(raydir))^2))
		local hit = false
		local nextdirx, nextdiry
		local actdistx, actdisty
		local dist
		local lineheight
		local side

		if (math.cos(raydir) < 0) then
			nextdirx = -1
			actdistx = ((x/tilesize) - mapx) * nextdistx
		else
			nextdirx = 1
			actdistx = (mapx + 1 - (x/tilesize)) * nextdistx
		end

		if (math.sin(raydir) < 0) then
			nextdiry = -1
			actdisty = ((y/tilesize) - mapy) * nextdisty
		else
			nextdiry = 1
			actdisty = (mapy + 1 - (y/tilesize)) * nextdisty
		end

		while hit == false do
			if (actdistx < actdisty) then
				actdistx = actdistx + nextdistx
				mapx = mapx + nextdirx
				side = 0
			else
				actdisty = actdisty + nextdisty
				mapy = mapy + nextdiry
				side = 1
			end
			for s = 1, #solidstable do
				if map[mapy][mapx] == solidstable[s] then 
					hit = true
				end
			end
		end

		if side == 0 then
			dist = math.abs((mapx - (x/tilesize) + (1 - nextdirx) / 2) / math.cos(raydir))
		else
			dist = math.abs((mapy - (y/tilesize) + (1 - nextdiry) / 2) / math.sin(raydir))
		end
	
		lineheight = math.abs(math.floor(wy / dist))
		draws = (wy - lineheight)/2
	
		if texture then
			local mapn = map[mapy][mapx]
			local wall

			if (side == 1) then 
				wall = x/tilesize + ((mapy - y/tilesize + (1 - nextdiry) / 2) / math.sin(raydir)) * math.cos(raydir)
			else 
				wall = y/tilesize + ((mapx - x/tilesize + (1 - nextdirx) / 2) / math.cos(raydir) * math.sin(raydir)) 
			end

			wall = wall - math.floor(wall)
		
			local tex = math.floor(wall * tilesize)

			if (side == 0 and math.cos(raydir) > 0) then tex = tilesize - tex - 1 end
			if (side == 1 and math.sin(raydir) < 0) then tex = tilesize - tex - 1 end
		
			local drawe = draws + lineheight
		
			local quad = love.graphics.newQuad(tex,0,1,tilesize,tilesize, tilesize)
			local texscale = (drawe - draws) / tilesize
			love.graphics.drawq(texture[mapn], quad, i, draws, 0, 1, texscale)
		else
			love.graphics.rectangle("fill",i, draws, 1, lineheight)
		end
	end
end

--[[raycasting collision: 0 == air 
		local nextx = x + (math.cos(dir) * speed) * dt
		local nexty = y + (math.sin(dir) * speed) * dt

		if map[math.floor(y/tilesize)][math.floor((nextx/tilesize)] == 0 then
			x = nextx
		end
		if map[math.floor((nexty)/tilesize)][math.floor(x/tilesize)] == 0 then
			y = nexty
		end
]]
