machinegun = {}

function machinegun:new(parent, delay, speed, knockback)
	local table = {}
	table.__index = machinegun
	table.delay = delay
	table.bullets = {}
	table.timer = timer:new(table.delay, false)
	table.speed = speed
	table.damage = 1
	table.knockback = knockback
	table.maxknockback = parent.maxyspeed/3
	table.parent = parent
	table.canshoot = true
	table.keyfix = true
	table.ammo = 100
	table.maxammo = table.ammo
	table.ammotimer = timer:new(0.2, true)
	table.img = love.graphics.newImage("res/machinegunbullet.png")	
	table.flash = {}
	table.flash.img = love.graphics.newImage("res/gunflash.png")
	table.flash.width = table.flash.img:getWidth()
	table.flash.height = table.flash.img:getHeight()
	table.width = table.img:getWidth()
	table.height = table.img:getHeight()
	table.view = love.graphics.newImage("res/machinegunview.png")
	table.has = false

	table.shell = {}
	local data = love.image.newImageData(1,1)
	data:setPixel(0,0,255,255,0,255)

	table.shell.img = love.graphics.newImage(data)
	table.shell.speed = 100
	table.shell.obj = {}

	setmetatable(table, table)
	return table
end

function machinegun:update(dt)
	for i = 1, #self.bullets do
		if self.bullets[i].active then	
			if self.bullets[i].dir == "left" then
				self.bullets[i].x = self.bullets[i].x - self.speed * dt
				self.bullets[i].y = self.bullets[i].y + math.sin(self.bullets[i].angl) * self.speed * dt
			elseif self.bullets[i].dir == "right" then
				self.bullets[i].x = self.bullets[i].x + self.speed * dt
				self.bullets[i].y = self.bullets[i].y + math.sin(self.bullets[i].angl) * self.speed * dt
			elseif self.bullets[i].dir == "up" then
				self.bullets[i].y = self.bullets[i].y - self.speed * dt
				self.bullets[i].x = self.bullets[i].x + math.sin(self.bullets[i].angl) * self.speed * dt
			else
				self.bullets[i].y = self.bullets[i].y + self.speed * dt
				self.bullets[i].x = self.bullets[i].x + math.sin(self.bullets[i].angl) * self.speed * dt
			end
			if self.bullets[i].x > wx - camera.x or self.bullets[i].x < 0 - camera.x then
				self.bullets[i].active = false
			end
			if self.bullets[i].y > wy - camera.y or self.bullets[i].y < 0 - camera.y then
				self.bullets[i].active = false
			end

			if self.bullets[i].dir == "left" then
				self.bullets[i].hitbox = {x = self.bullets[i].x - self.bullets[i].width + self.bullets[i].width/2, y = self.bullets[i].y - self.bullets[i].height, width = self.bullets[i].width/2, height = self.bullets[i].height}
			elseif self.bullets[i].dir == "right" then
				self.bullets[i].hitbox = {x = self.bullets[i].x, y = self.bullets[i].y, width = self.bullets[i].width/2, height = self.bullets[i].height}
			elseif self.bullets[i].dir == "up" then
				self.bullets[i].hitbox = {x = self.bullets[i].x, y = self.bullets[i].y - self.bullets[i].height + self.bullets[i].height/2, width = self.bullets[i].width, height = self.bullets[i].height/2}
			else
				self.bullets[i].hitbox = {x = self.bullets[i].x, y = self.bullets[i].y, width = self.bullets[i].width, height = self.bullets[i].height/2}
			end
			
			self:collision(i)	
		
			if self.bullets[i].active == false then
				self.bullets[i] = nil
			end
		
		end
	end
	
	for i = 1,#self.shell.obj do
		local o = self.shell.obj[i]
		o.x = o.x + o.velx * dt
		o.y = o.y + o.vely * dt

		o.vely = o.vely + gravity * dt
		

		-- collision
	
		local tile = map:getneartilepos(o.x, o.y, 1, 1, currentmap)	
		local hitbox = {x = o.x, y= o.y, width = 1, height = 1}
		for k = 1, #tile do
			for y = 1, #solidids do
				if tile[k].id == solidids[y] then
					local col = getcollision(hitbox, tile[k],1)
					if col then
						if col.ver then
							o.x = o.x + col.x
							o.y = o.y + col.y
							if col.side == "up" then
								o.vely = 0
							else
								o.vely = 0
								o.velx = 0
							end
						end
						if col.hor then
							o.x = o.x + col.x
							o.y = o.y + col.y
							o.velx = 0	
						end						

					end
				end
			end
		end

		if o.timer.tick then
			self.shell.obj[i] = nil
		end

	end

	ordertable(self.bullets)
	ordertable(self.shell.obj)

	if self.canshoot == false then
		self.timer.active = true
	end
	if self.timer.tick then
		self.timer.active = false
		self.timer.tick = false
		self.canshoot = true
	end
	
	if self.ammotimer.tick and self.canshoot then
		if key.actiondown == false then
			self.ammo = self.ammo + 1
			if self.ammo >= self.maxammo then
				self.ammo = self.maxammo
			end
		end
	end


end

function machinegun:draw()
	for i = 1, #self.bullets do
		if self.bullets[i].active then
			local scax = 1
			local scay = 1
			local rad = 0
			if self.bullets[i].dir == "left" then
				scax = -1
				scay = -1
			elseif self.bullets[i].dir == "right" then
				scax = 1
			elseif self.bullets[i].dir == "down" then
				rad = -math.pi/2
				scax = -1
				scay = 1
			else 
				rad = math.pi/2
				scax = -1
				scay = -1
			end
			love.graphics.draw(self.img, self.bullets[i].x, self.bullets[i].y,rad,scax,scay)

			if self.bullets[i].flash then
				local x,y
				if self.bullets[i].dir == "left" then
					scax = -1
					scay = -1	
					x = self.bullets[i].x - self.width
					y = self.bullets[i].y + self.flash.height/2 - self.height/2
				elseif self.bullets[i].dir == "right" then
					scax = 1
					x = self.bullets[i].x + self.flash.width/2 - self.width/2
					y = self.bullets[i].y - self.flash.height/2 + self.height/2
				elseif self.bullets[i].dir == "down" then
					rad = -math.pi/2
					scax = -1
					scay = 1
					x = self.bullets[i].x - self.flash.width/2 + self.width/2
					y = self.bullets[i].y + self.height
				else 
					rad = math.pi/2
					scax = -1
					scay = -1
					x = self.bullets[i].x - self.flash.width/2 + self.width/2
					y = self.bullets[i].y - self.flash.height/2 - self.height/2
				end
				love.graphics.draw(self.flash.img, x, y,rad,scax,scay)
				self.bullets[i].flash = false
			end

			if gamedebug then
				love.graphics.rectangle("line", self.bullets[i].hitbox.x , self.bullets[i].hitbox.y, self.bullets[i].hitbox.width, self.bullets[i].hitbox.height)
			end
		end
	end
	for i = 1,#self.shell.obj do
		local o = self.shell.obj[i]
		love.graphics.draw(self.shell.img,o.x,o.y)
	end
end

function machinegun:shoot()
	if self.ammo <= 0 then
		self.canshoot = false
	end
	
	if self.canshoot then
		local tab = {}
		tab.active = true
		tab.dir = self.parent.dir
		tab.angl = math.rad(math.random(-2,2))
		tab.flash = true
		self.ammo = self.ammo - 1
		if self.parent.look == "rect" then
			tab.width = self.width
			tab.height = self.height
			tab.x = self.parent.x + self.parent.width/2 + tab.width/2
			tab.y = self.parent.y + self.parent.height/2 - tab.height/2
			if tab.dir == "left" then
				tab.x = self.parent.x + self.parent.width/2 - tab.width/2
				tab.y = tab.y + tab.height
				tab.x = tab.x + tab.width
			end
		else
			tab.width = self.height
			tab.height = self.width
			tab.x = self.parent.x + self.parent.width/2
			tab.y = self.parent.y + self.parent.height/2 - tab.height/2
			if self.parent.look == "up" then
				tab.dir = "up"	
				tab.y = tab.y + tab.height
			else
				tab.dir = "down"
			end
		end
		self.bullets[#self.bullets + 1] = tab
		self.canshoot = false
		if self.parent.look == "down" then
			if self.ammo > 1 then
				local dt = love.timer.getDelta()
				if self.parent.vely > 0 then
					self.parent.vely = 0
				end
				self.parent.vely = self.parent.vely - self.knockback * dt
				if self.parent.vely < - self.maxknockback then
					self.parent.vely = - self.maxknockback
				end
			end
		end
		local obj = {}
		obj.x = tab.x
		obj.y = tab.y
		if self.parent.dir == "right" then
			obj.angle = -math.rad(math.random(90, 180))
		else
			obj.angle = -math.rad(math.random(0, 90))
		end
		obj.velx = math.cos(obj.angle) * self.shell.speed
		obj.vely = math.sin(obj.angle) * self.shell.speed
		obj.timer = timer:new(10, true)

		self.shell.obj[#self.shell.obj + 1] = obj
	end
end

function machinegun:collision(i)
	local tile = map:getneartilepos(self.bullets[i].hitbox.x, self.bullets[i].hitbox.y, self.bullets[i].hitbox.width, self.bullets[i].hitbox.height, currentmap)	
	for k = 1, #tile do
		for y = 1, #solidids do
			if tile[k].id == solidids[y] then
				if shortaabb(self.bullets[i].hitbox, tile[k]) then
					self.bullets[i].active = false

					local width,height = collisionimg:getWidth()/5,collisionimg:getHeight()
					particles[#particles +1] = partsys:new(collisionimg, self.bullets[i].x - width/2, self.bullets[i].y - height/2, 0.5, 0)
					
				end
				break
			end
		end
	end
	---- enemy collision
	for y = 1, #enemy do
		for k = 1, #enemy[y] do
			if enemy[y][k].active then
				if shortaabb(enemy[y][k].hitbox ,self.bullets[i].hitbox) then
					local dragonfire = false

					if enemy[y][k].firetimer ~= nil then -- dragon
						if enemy[y][k].firetimer.active then
							dragonfire = not enemy[y][k].hitboxchange	
						end
					end
		
					if dragonfire == false then
						enemy[y][k].health = enemy[y][k].health - self.damage
						enemy[y][k].hit = true
						self.bullets[i].active = false
						if enemy[y][k].health <= 0 then
							enemy[y][k].dead = true
						end	
					

						local num = math.random(3,5)
						local angle = {}
						local dir = self.bullets[i].dir
						for u = 1, num do
							if dir == "right" then
								angle[u] = math.random(90, 270)
							elseif dir == "left" then
								angle[u] = math.random(-90, 90)
							elseif dir == "up" then
								angle[u] = math.random(180, 360)
							else
								angle[u] = math.random(0, 180)
							end
						end
						particles[#particles +1] = partsys:new(bloodimg, self.bullets[i].x, self.bullets[i].y, 0.5, {tilesize,1.2 * tilesize}, num, angle)
						break
					end
				end
			end
		end
	end
end

