
_getdt = love.timer.getDelta

function love.timer.getDelta()
	local dt = _getdt()
	if dt > 1/30 then
		dt = 1/30
	end
	return dt
end

jit.on()

map = require("libs/map")
camera = require("libs/camera")
sprite = require("libs/sprite")
timer = require("libs/timer")
partsys = require("libs/particlesys")
json = require("libs/dkjson")
require("libs/physics")
require("gun")
require("bomb")
require("machinegun")
require("gem")
require("game")
require("menu")
require("gui")
require("powerup")
require("player") 
require("spike")
require("enemy/blob")
require("enemy/redblob")
require("enemy/blueblob")
require("enemy/fly")
require("enemy/boss1")
require("enemy/boss2")	
require("enemy/boss3")
require("loadmaps")

function love.load() 
	wx = 320
	wy = 240
	wratio = wx/wy
	scale = 2.8
	tilesize = 16
	gamedebug = false
	love.window.setMode(wx*scale,wy*scale,{vsync = true})	
	love.graphics.setDefaultFilter("nearest", "nearest", 0)
	love.graphics.setLineStyle("rough")

	pony = true

	if pony then
		dir = "pony"
	else
		dir = "res"
	end

	menu:init()
	game:init()


	state = menu

	key = {}
	key.up = "up"
	key.updown = false
	key.right = "right"
	key.rightdown = false
	key.left = "left"
	key.leftdown = false
	key.down = "down"
	key.downdown = false
	key.action = "x"
	key.actiondown = false
	key.jump = "z"
	key.jumpdown = false
	key.upgun = "s"
	key.upgundown = false
	key.upgunpress = false
	key.downgun = "a"
	key.downgundown = false
	key.downgunpress = false
end

function love.update(dt)

	timer:update(dt)
	state:update(dt)

	if key.upgundown then
		key.upgundown = false
		key.upgunpress = false
	end

	if key.downgundown then	
		key.downgundown = false
		key.downgunpress = false	
	end
end

function love.draw()
	love.graphics.scale(scale)

		state:draw()

end

function love.keypressed(k)
	state:keypressed(k)	
	
	if k == key.up then
		key.updown = true
	end
	if k == key.right then	
		key.rightdown = true
	end
	if k == key.left then
		key.leftdown = true
	end
	if k == key.down then
		key.downdown = true
	end
	if k == key.jump then
		key.jumpdown = true
	end
	if k == key.action then
		key.actiondown = true
	end
	if k == key.upgun and key.upgunpress == false then
		key.upgundown = true
		key.upgunpress = true
	end
	if k == key.downgun and key.downgunpress == false then
		key.downgundown = true
		key.downgunpress = true
	end

	----------------------------------
	if k == "escape" then
		love.event.quit()
	end
end

function love.keyreleased(k)
	if k == key.up then
		key.updown = false
	end
	if k == key.right then	
		key.rightdown = false
	end
	if k == key.left then
		key.leftdown = false
	end
	if k == key.down then
		key.downdown = false
	end
	if k == key.jump then
		key.jumpdown = false
	end
	if k == key.action then
		key.actiondown = false
		player.currentgun.keyfix = true
	end
end

function copymap(t)
	local twidth = #t[1]
	local theight = #t
	local newt = {}
	for i = 1, theight do
		newt[i] = {}
	end
	for i = 1, theight do
		for y = 1, twidth do
			newt[i][y] = t[i][y]
		end
	end
	return newt
end
