player = {}
player.__index = player

function player:init(x, y)
	self.x = x
	self.y = y
	self.velx = 0
	self.vely = 0
	self.platvel = 0
	self.gravity = gravity
	self.speed = 500
	self.jumpspeed = 110
	self.jumptimer = timer:new(0.3,false)
	self.widthdif = 4
	self.heightdif = 2
	self.width = tilesize - self.widthdif
	self.height = tilesize - self.heightdif
	self.xoffset = self.widthdif/2
	self.yoffset = self.heightdif/2
	self.hitbox = {x = self.x + self.xoffset, y = self.y + self.yoffset, width = self.width, height = self.height}
	self.maxxspeed = 100
	self.maxyspeed = 200
	self.state = "stand"
	self.dir = "right"
	self.look = "rect"
	self.dead = false
	self.anim = sprite:new("res/sprite.png", tilesize, tilesize, {{"stand", 1}, {"run", 4},{"down",1},{"up",1},{"jump", 1},{"jumpdown",1},{"jumpup",1}, {"atack", 1},{"atackup",1},{"jumpatack",1},{"jumpatackdown",1},{"jumpatackup",1},{"runatack",4},{"runatackup",4}},0,self.yoffset)
	self.friction = false
	self.onground = false
	self.score = 0
	self.gems = 0
	self.hp = 50
	self.maxhp = self.hp
	self.gun = {}
	self.gun.num = 0
	self.gun.own = 0
	self.gun["simple"] = gun:new(self, 0,200)
	self.gun["bombgun"] = bombgun:new(self, 0.8, 150)
	self.gun["machine"] = machinegun:new(self, 0.1, 300, 7800)


	self.gun["init"] = {}
	self.gun["init"].view = self.gun["simple"].view

	self.currentgun = self.gun["init"]

	self.atacktimer = timer:new(0.2, true) -- atack state

	self.mercy = false -- mercy invisibility(noob proteccion)
	self.mercytimer = timer:new(1.8, false)
	self.mercyflash = false
	self.pushback = 300 -- from enemy

	setmetatable(self,self)
end

function player:update(dt)
	if self.dead == false then
		self:move(dt)
		self:collision()
		self:states(dt)
		
		self.gun["simple"]:update(dt)
		self.gun["bombgun"]:update(dt)
		self.gun["machine"]:update(dt)

		if self.hp < 1 then
			self.dead = true
			self.hp = 0
			deadtimer.active = true
		end
	
		if self.mercy then
			self.mercyflash = not self.mercyflash
			if self.mercytimer.tick then
				self.mercytimer.active = false
				self.mercy = false
				self.mercyflash = false
			end
				
		end
	end
end

function player:draw()
	if self.dead == false then
		if gamedebug == true then
			love.graphics.setColor(0, 255, 0)
			love.graphics.rectangle("line", self.hitbox.x, self.hitbox.y, self.hitbox.width, self.hitbox.height)
		end
		love.graphics.setColor(255,255,255)
		
		if self.mercyflash == false then
			self.anim:draw(self, 0.2)
		end
		self.gun["simple"]:draw()
		self.gun["bombgun"]:draw()
		self.gun["machine"]:draw()
	end
end

function player:states(dt)
	if self.atacktimer.active == false then	
		local fix = self.speed * dt
		if self.velx == 0 and self.vely == 0 then
			self.state ="stand"
			if self.look == "up" then
				self.state = "up"
			elseif self.look == "down" then
				self.state = "down"
			end
		end
		if self.vely > fix then
			self.state = "jump"
			self.onground = false
			if self.look == "up" then
				self.state = "jumpup"
			elseif self.look == "down" then
				self.state = "jumpdown"
			end
			
		end
		if self.onground == false then
			self.state = "jump"
			if self.look == "up" then
				self.state = "jumpup"
			elseif self.look == "down" then
				self.state = "jumpdown"
			end
		end
	else
		self.state = "atack"
			
		if self.velx ~=0 then
			self.state = "runatack"
		end
		if self.onground == false then
			self.state = "jumpatack"
		end
		if self.look == "up" then
			self.state = "atackup"
			if self.velx ~=0 then
				self.state = "runatackup"
			end
			if self.onground == false then
				self.state = "jumpatackup"
			end
		elseif self.look == "down" and self.onground == false then
			self.state = "jumpatackdown"
		end
		if self.atacktimer.tick then
			self.atacktimer.active = false
		end
	end
end

function player:move(dt)
	self:control(dt)

	self.x = self.x + (self.velx + self.platvel) * dt
	self.y = self.y + self.vely * dt

	self.vely = self.vely + self.gravity * dt

	if self.velx > self.maxxspeed then -- speed max limitation
		self.velx = self.maxxspeed
	elseif self.velx < -self.maxxspeed then
		self.velx = -self.maxxspeed
	end
	if self.vely > self.maxyspeed then
		self.vely = self.maxyspeed
	elseif self.vely < -self.maxyspeed then
		self.vely = -self.maxyspeed
	end
	
	if self.friction then -- friction stuff
		local frictionconstant = 140*dt
		local newfriction
		if self.onground == false then
			newfriction = friction/5 -- onair friction
		else
			newfriction = friction
		end
		if self.velx < 0 then
			self.velx = self.velx + newfriction * dt
		elseif self.velx > 0 then
			self.velx = self.velx - newfriction * dt
		end
	
		if self.velx < frictionconstant and self.velx > -frictionconstant then
			self.velx = 0
		end
	end

	if self.onground == false then
		self.platvel = 0
	end
end

function player:control(dt)
	self.look = "rect"

	if self.gun.own > 0 then
		if key.downgundown then
			self.gun.num = self.gun.num - 1
			if self.gun.num < 1 then
				self.gun.num = self.gun.own
			end
		end
		if key.upgundown then
			self.gun.num = self.gun.num + 1
			if self.gun.num > self.gun.own then
				self.gun.num = 1
			end
		end

		if self.gun.num == 1 then
			self.currentgun = self.gun["simple"]
		elseif self.gun.num == 2 then
			self.currentgun = self.gun["bombgun"]
		elseif self.gun.num == 3 then
			self.currentgun = self.gun["machine"]
		end
	end

	if key.updown then
		self.look = "up"
	end
	if key.downdown then
		self.look = "down"
	end	
	if key.rightdown then
		self.velx = self.velx + self.speed * dt
		self.dir = "right"
	end
	if key.leftdown then
		self.velx = self.velx - self.speed * dt
		self.dir = "left"
	end
	if key.jumpdown then
		if self.onground then
			self.onground = false
			self.jumptimer.active = true
			self.jumptimer.time = 0
			self.vely = self.vely - self.jumpspeed
		end
		if self.jumptimer.tick then
			self.jumptimer.active = false
			self.gravity = gravity
			self.jumptimer.time = 0
		end
		if self.jumptimer.active then
			self.gravity = 0
		end
	else
		if self.jumptimer.active then
			self.jumptimer.active = false
			self.jumptimer.time = 0
			self.gravity = gravity
		end
	end
	if key.actiondown and self.currentgun ~= self.gun["init"] then
		if self.currentgun.keyfix then
				if self.look == "down" and self.onground then
					self.look = "rect"
				end
				self.atacktimer.active = true
				self.atacktimer.time = 0
				if self.gun.own <= 3 then
					self.currentgun:shoot()
				end
		end
	end

	if key.leftdown or key.rightdown then
		self.friction = false
	else
		self.friction = true
	end
end

function player:collision()
	self.hitbox = {x = self.x + self.xoffset, y = self.y + self.yoffset, width = self.width, height = self.height}

	for y = 1, #enemy do
		for k = 1, #enemy[y] do
			if enemy[y][k].active then
				local col = getcollision(self.hitbox, enemy[y][k].hitbox)
				if enemy[y][k].type == "boss" then
					if self.mercy then
						col = false
					end
				end
				if col then
					if col.hor then
						self.x = self.x + col.x 
						self.velx = 0
					end
					if col.ver then
						if col.y < 0 then
							self.y = self.y + col.y
						end
						self.vely = 0
						if col.side == "down" then
							if self.atacktimer.active == false then
								self.state = "run"
							end
							self.onground = true	
							self.platvel = enemy[y][k].velx
						end
						if col.side == "up" then
							self.vely = 0
						end
					end
					
					if self.mercy == false then
						self.hp = self.hp - enemy[y][k].damage		
						self.mercy = true
						self.mercytimer.active = true
						if col.side == "right" then
							self.velx = -self.pushback
							self.vely = -50
						elseif col.side == "left" then
							self.velx = self.pushback
							self.vely = -50
						elseif col.side == "down" then
							self.vely = -50
						elseif col.side == "up" then
							if col.x > 0 then
								self.velx = -self.pushback
							else
								self.velx = self.pushback
							end
						end
						self.onground = false
					end

				end
			end
		end
	end

	local tile = map:getneartilepos(self.x, self.y, self.width, self.height, currentmap)	
	for i = 1, #tile do
		for y = 1, #solidids do
			if tile[i].id == solidids[y] then
				local col = getcollision(self.hitbox, tile[i])
				if col then
					if col.hor then
						self.x = self.x + col.x 
						self.velx = 0
					end
					if col.ver then
						self.y = self.y + col.y
						self.vely = 0
					end
					if col.side == "down" then
						if self.atacktimer.active == false then
							self.state = "run"
						end
						self.onground = true
					end
					if col.side == "up" then
						if self.jumptimer.active then
							self.jumptimer.active = false

							self.jumptimer.time = 0
							self.gravity = gravity

						end
					end
					self.hitbox = {x = self.x + self.xoffset, y = self.y + self.yoffset, width = self.width, height = self.height}
				end
			end
		end
	end
end

